@@ -126,7 +126,8 @@ function(database = "world", regions = ".", exact = FALSE,
 	 col = 1, plot = TRUE, add = FALSE, namesonly = FALSE, 
          xlim = NULL, ylim = NULL, wrap = FALSE,
          resolution = if (plot) 1 else 0, type = "l", bg = par("bg"),
-         mar = c(4.1, 4.1, par("mar")[3], 0.1), myborder = 0.01, namefield="name", ...)
+         mar = c(4.1, 4.1, par("mar")[3], 0.1), myborder = 0.01, 
+         namefield="name", lforce = "n", ...)
 {
   # parameter checks
   if (resolution>0 && !plot) 
@@ -135,20 +136,39 @@ function(database = "world", regions = ".", exact = FALSE,
     stop("one of boundary and interior must be TRUE")
   doproj <- !missing(projection) || !missing(parameters) || !missing(
           orientation)
+  if (doproj && !requireNamespace("mapproj", quietly=TRUE)) {
+    stop("Please install the package 'mapproj' for projections.")
+  }
   coordtype <- maptype(database)
   if (coordtype == "unknown") 
      stop("missing database or unknown coordinate type")
   if (doproj && coordtype != "spherical") 
     stop(paste(database, "database is not spherical; projections not allowed"))
+  if (length(wrap)>=2 && !doproj && wrap[2] - wrap[1] != 360) 
+    stop("The specified longitudes for wrapping are inconsistent, they should be 360 apart.")
   # turn the region names into x and y coordinates
   if (is.character(database)) as.polygon = fill
-  else as.polygon = TRUE
-  coord <- map.poly(database, regions, exact, xlim, ylim, 
+  else as.polygon <- TRUE
+  # if we are going to wrap around anything else than [-180, 180], the simplest way
+  # to get all necessary polylines/polygons is to set xlim=NULL *temporarily*
+  # (alternatively, the C code must shift longitudes by +/- 360 to fit in boundaries)
+  xlim_tmp <- if (length(wrap)>=2) NULL else xlim
+  coord <- map.poly(database, regions, exact, xlim_tmp, ylim, 
                     boundary, interior, fill, as.polygon, namefield=namefield)
-  if (is.na(coord$x[1])) stop("first coordinate is NA.  bad map data?")
+  if (is.na(coord$x[1])) stop("first coordinate is NA. Bad map data?")
+  if (length(wrap)>=2) {
+    antarctica <- if (length(wrap) == 2) -89.9 else wrap[3]
+    coord <- map.wrap.poly(coord, xlim=wrap[1:2], poly=fill, antarctica=antarctica)
+  }
+  # we can enforce xlim & ylim exactly
+  # this changes the output
+  if (lforce=="e") {
+    coord <- map.clip.poly(coord, xlim, ylim, poly=fill)
+  }
   if (plot) {
     .map.range(coord$range)
   }
+
   if (doproj) {
     nam <- coord$names
     coord <- mapproj::mapproject(coord, projection = projection,
@@ -156,15 +176,25 @@ function(database = "world", regions = ".", exact = FALSE,
     coord$projection = projection
     coord$parameters = parameters
     coord$orientation = orientation
+    coord$names <- nam
+    if (!is.null(xlim) && !is.null(ylim) && lforce %in% c("s","l")) {
+      prange <- mapproj::mapproject(x=rep(xlim,2), y=rep(ylim, each=2))
+      if (lforce=="s") {
+        xlim <- c(max(prange$x[c(1,3)]),min(prange$x[c(2,4)]))
+        ylim <- c(max(prange$y[c(1,2)]),min(prange$y[c(3,4)]))
+      } else {
+        xlim <- c(min(prange$x[c(1,3)]),max(prange$x[c(2,4)]))
+        ylim <- c(min(prange$y[c(1,2)]),max(prange$y[c(3,4)]))
+      }
+    }
     if (plot && coord$error)
       if (all(is.na(coord$x)))
         stop("projection failed for all data")
       else warning("projection failed for some data")
-    coord$names <- nam
   }
   # AD: we do wrapping first: slightly better than when run after the thinning
   #     also now the output data is also wrapped if plot=FALSE
-  if (wrap) coord <- map.wrap(coord)
+  if (length(wrap)==1 && wrap) coord <- map.wrap(coord)
   # do the plotting, if requested
   if (plot) {
     # for new plots, set up the coordinate system;
@@ -174,10 +204,10 @@ function(database = "world", regions = ".", exact = FALSE,
     if (!add) {
       opar = par(bg = bg)
       if (!par("new")) plot.new()
-      # xlim, ylim apply before projection
-      if (is.null(xlim) || doproj) xrange <- range(coord$x, na.rm = TRUE)
+      # xlim, ylim apply before projection unless we have pforce=TRUE
+      if (is.null(xlim) || (doproj && !(lforce %in% c("s","l")))) xrange <- range(coord$x, na.rm = TRUE)
       else xrange <- xlim
-      if (is.null(ylim) || doproj) yrange <- range(coord$y, na.rm = TRUE)
+      if (is.null(ylim) || (doproj && !(lforce %in% c("s","l")))) yrange <- range(coord$y, na.rm = TRUE)
       else yrange <- ylim
       if (coordtype != "spherical" || doproj) {
 	aspect <- c(1, 1) 
